// TODO: recal/render voting grid on resize
// https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver


// The following needs to be kept in sync with the .voting-countries & .voting-points widths in css/eurovision.css
var glob_voting_country_and_points_width = (180+35);  


var pp_country_lookup = {
    "BosniaandHerzegovina":        "Bosnia and Herzegovina",
    "CzechRepublic":               "Czech Republic",
    "FederalRepublicofYugoslavia": "Federal Republic of Yugoslavia",
    "NorthMacedonia":              "North Macedonia",
    "RepublicofMacedonia":         "Republic of Macedonia",
    "SanMarino":                   "San Marino",
    "SerbiaandMontenegro":         "Serbia and Montenegro", 
    "UnitedKingdom":               "United Kingdom",
};


var esc_countries = [
    "Albania",
    "Andorra",
    "Armenia",
    "Australia",
    "Austria",
    "Azerbaijan",
    "Belarus",
    "Belgium",
    "Bosnia and Herzegovina",
    "Bulgaria",
    "Croatia",
    "Cyprus",
    "Czech Republic",
    "Denmark",
    "Estonia",
    "Finland",
    "France",
    "Georgia",
    "Germany",
    "Greece",
    "Hungary",
    "Iceland",
    "Ireland",
    "Israel",
    "Italy",
    "Latvia",
    "Lithuania",
    "Luxembourg",
    "Macedonia",
    "Malta",
    "Moldova",
    "Monaco",
    "Montenegro",
    "Morocco",
    "The Netherlands",
    "North Macedonia",
    "Norway",
    "Poland",
    "Portugal",
    "Romania",
    "Russia",
    "San Marino",
    "Serbia",
    "Serbia and Montenegro",
    "Slovakia",
    "Slovenia",
    "Spain",
    "Sweden",
    "Switzerland",
    "Turkey",
    "Ukraine",
    "United Kingdom",
    "Yugoslavia",
];




function voting_mode(year)
{

    var vote_mode = undefined;

    if (year>=2013) {
	// Separate voting: Jury and Tele 
        vote_mode = "Separated";
    }
    else if ((year>=2001) && (year<=2012)) {
        vote_mode = "Combined";
    }
    else {
        vote_mode = "Single";
    }
    
    return vote_mode;
}


function nuanced_voting_type(country,year)
{
    var vote_type = undefined;

    if (year>=2013) {
        // Separate voting: Jury and Tele 
        vote_type = "JT";
    }
    else if ((year>=2009) && (year<=2012)) {
        if (country=="San Marino") {
            vote_type = "J";
	}
        else {
            // Combined Jury and Tele
            vote_type = "C";
	}
    }
    else if ((year>=2003) && (year<=2008)) {
        vote_type = "T";
    }
    else if (year==2002) {
        var jury_list = [ "Bosnia and Herzegovina", "Turkey", "Russia", "FYR Macedonia", "Romania"];
        var comb_list = [ "Cyprus", "Greece", "Spain", "Croatia", "Finland", "Malta", "Slovenia", "Lithuania" ];
        if (country in jury_list) {
            vote_type = "J";
	}
        else if (country in comb_list) {
            vote_type = "C";
	}
        else {
            vote_type = "T";
	}
    }
    else if (year==2001) {
        var jury_list = [ "Bosnia and Herzegovina", "Turkey", "Russia" ];
        var comb_list = [ "Croatia", "Greece", "Malta" ];
        if (country in jury_list) {
            vote_type = "J";
	}
        else if (country in comb_list) {
            vote_type = "C";
	}
        else {
            vote_type = "T";
	}
    }
    else if ((year>=1998) && (year<=2000)) {
        vote_type = "T";
    }
    else if (year==1997) {
        var tele_list = [ "Austria", "Germany", "Sweeden", "Switzerland", "United Kingdom" ];
        if (country in tele_list) {
            vote_type = "T";
	}
        else {
            vote_type = "J";
	}
    }
    else {
        vote_type = "J";
    }

    return vote_type;
}



function create_array2d(num_cols,num_rows)
{
    var array_2d = new Array(num_rows);

    for (var i = 0; i < array_2d.length; i++) {
	array_2d[i] = new Array(num_cols);
    }

    return array_2d;
}

function create_i_to_yx_mapping(num_cols,num_rows,num_countries)
{
    var entries_overflow = (num_countries % num_cols)
    var num_entries_in_last_row = (entries_overflow == 0) ? num_cols : entries_overflow;
    
    var sorted_results_mapping = create_array2d(num_cols,num_rows);
    
    // populate the 2d-array with the suitable mapped in values
    var i = 0;
    for (var x=0; x<num_cols; x++) {
	for (var y=0; y<num_rows; y++) {
	    
	    var on_last_row = (y == (num_rows-1));
	    var in_last_row_gap = (on_last_row && (x >= num_entries_in_last_row));
	    
	    if (!in_last_row_gap) {
		sorted_results_mapping[y][x] = i;
		i++;
	    }
	    else {
		sorted_results_mapping[y][x] = undefined;
	    }
	    
	}
    }

    return sorted_results_mapping;
}


function transpose_iDEPRECATED(i,num_cols,num_rows,num_countries)
{
    if (i >= num_countries) {
	// out of range to tanspose, just return original value
	return i;
    }
    
    var num_filler_gaps = (num_cols * num_rows) - num_countries;
    var full_cols_until = num_cols - num_filler_gaps;
    //console.log("full cols until = " + full_cols_until);
    
    // Map 'i' to (x,y) position in grid layout
    var x = (i % num_cols);
    var y = Math.floor(i / num_cols); // effectively int division

    // Transpose, and then reverse engineer what value of 'i' the transposed position needs to be
    var trans_x = y;
    var trans_y = x;
    
    var trans_i = trans_y * num_rows + trans_x;

    // Filler correction
    //if ((x > full_cols_until) && (trans_y < (num_cols-1))) {
    if (x > full_cols_until) {
	trans_i = trans_i - (num_cols - x)
    }
    
    return trans_i;
}

function append_to_country_votes(vote_label, votes_json_type, votes_type_total)
{
    // TODO
    // Figure out why "United Kingdom-J" and not "UnitedKingdom-J"

    if (gs.documentMetadata[votes_json_type]) {
	var all_votes_div_width  = $('#to-country-votes').width();
	
	var $vote_label_div = $('<div>').attr("class","voting-label").append(vote_label);
	$vote_label_div.append(" (Total: " + gs.documentMetadata[votes_type_total] + ")");
	$('#to-country-votes').append($vote_label_div);

	var $vote_points_div = $('<div>').attr("class","voting-points-div");
	
	var VotesJSON = eval(gs.documentMetadata[votes_json_type]);
	var num_countries = VotesJSON.length;

	var num_cols = Math.floor(all_votes_div_width / glob_voting_country_and_points_width);
	var num_rows = Math.ceil(num_countries / num_cols);
	var num_cells = num_cols * num_rows;
	
	VotesJSON.sort();

	var library_name = gs.xsltParams["library_name"];
	var coll_name = gs.cgiParams["c"];
	
	var collect_doc_prefix_url = library_name + "/collection/" + coll_name + "/document/";

	var sorted_results_mapping = create_i_to_yx_mapping(num_cols,num_rows,num_countries)
	
	for (var y=0; y<num_rows; y++) {
	    for (var x=0; x<num_cols; x++) {
	
		var i = sorted_results_mapping[y][x];
		
		// By default, the country and points are set to be empty divs
		var $to_country_name = $('<div>').attr("class","voting voting-country").append("");
		var $to_country_points = $('<div>').attr("class","voting voting-points").append("");
		var $a_from_country = $('<span>');
	    
		// if 'i' explicitly undefined, we are in one of the last-row gaps  => nothing to do!
		if (typeof i !== 'undefined') {
		    var from_country  = VotesJSON[i];
		    
		    // map -C and -S to -J for now
		    from_country = from_country.replace(/-C$/,"-J");
		    from_country = from_country.replace(/-S$/,"-J");
		    
		    var from_country_points = gs.documentMetadata[from_country];
		    var year = gs.documentMetadata["Year"];
		
		    if (typeof from_country_points !== 'undefined') {
			var from_country_base = from_country.replace(/-(T|J)$/,"");

			var pp_from_country = from_country_base;
			if (pp_country_lookup[pp_from_country]) {
			    pp_from_country = pp_country_lookup[pp_from_country];
			}

			var hyphen_vote_type = from_country.match(/-(?:T|J)$/);
			var from_country_id = "FromCountry-"+from_country_base + year + hyphen_vote_type;
			var href_from_country = collect_doc_prefix_url + from_country_id;
			
			//var $a_from_country = $('<a>').attr("href",href_from_country).append(pp_from_country);
			var $a_from_country = $('<a>').attr("href",href_from_country);
		    
			//$from_country_name = $('<div>').attr("class","voting voting-country").append($a_from_country).append(":")
			$from_country_name = $('<div>').attr("class","voting voting-country").append(pp_from_country).append(":");
			$from_country_points = $('<div>').attr("class","voting voting-points").append(from_country_points);
		    }
		    else {
			console.error("Failed to find points for: " + from_country);
		    }
		}

		$a_from_country
		    .append($from_country_name)
		    .append($from_country_points);
		$('#to-country-votes')
		    .append($a_from_country);
	    }
		
	    $('#to-country-votes')
		.append("<br />");
	}

	//$('#to-country-votes').append($vote_points_div);

    }
}
	  
function append_all_to_country_votes()
{
    $('#to-country-votes').html("");
    
    append_to_country_votes("Jury Votes","JuryVotesJSON","JuryVotesTotal");
    append_to_country_votes("Televotes","TeleVotesJSON","TeleVotesTotal");
    
    append_to_country_votes("Combined Jury & Tele Votes","CombVotesJSON","CombVotesTotal");
    append_to_country_votes("Voting Exclusively by Jury","SingVotesJSON","SingVotesTotal");
}

function compare_countries(a,b)
{
    if (a.Country < b.Country) {
	return -1;
    }
    if (a.Country > b.Country) {
	return 1;
    }

    // names must be equal
    return 0;
}

function compare_points(a,b)
{
    return a.Points - b.Points;
}

function append_from_country_votes(vote_type)
{
    if (vote_type != "") {
	var all_votes_div_width  = $('#from-country-votes').width();
	
	var votes_country_json_type = "VotesCountryJSON-" + vote_type;
	var votes_points_json_type = "VotesPointsJSON-" + vote_type;
	var vote_label;
	if (vote_type == "J") {
	    vote_label = "Jury votes";
	}
	else if (vote_type == "T") {
	    vote_label = "Televotes";
	}
	else {
	    vote_label = "Separate Jury and Televotes Added Together";
	}
	
	var $vote_div = $('<div>').attr("class","voting-label").append(vote_label);
	$('#from-country-votes').append($vote_div);
	
	var VotesCountryJSON = eval(gs.documentMetadata[votes_country_json_type]); // map JSON string into data-structure
	var VotesPointsJSON = eval(gs.documentMetadata[votes_points_json_type]);   // map JSON string into data-structure

	var num_countries = VotesCountryJSON.length;
	
	var num_cols = Math.floor(all_votes_div_width / glob_voting_country_and_points_width);
	var num_rows = Math.ceil(num_countries / num_cols);
	var num_cells = num_cols * num_rows;
	
	var from_country_votes = [];
	
	for (var i=0; i<num_countries; i++) {
	    from_country_votes.push({ "Country" : VotesCountryJSON[i], "Points" : VotesPointsJSON[i]})
	}

	from_country_votes.sort(compare_countries);

	var library_name = gs.xsltParams["library_name"];
	var coll_name = gs.cgiParams["c"];
	
	var collect_doc_prefix_url = library_name + "/collection/" + coll_name + "/document/";

	var year = gs.documentMetadata["FromCountryYear"];

	var sorted_results_mapping = create_i_to_yx_mapping(num_cols,num_rows,num_countries)

	for (var y=0; y<num_rows; y++) {
	    for (var x=0; x<num_cols; x++) {
	
		var i = sorted_results_mapping[y][x];
		
		var $to_country_name = $('<div>').attr("class","voting voting-country").append("");
		var $to_country_points = $('<div>').attr("class","voting voting-points").append("");
		var $a_to_country = $('<span>');

		// if 'i' explicitly undefined, we are in one of the last-row gaps  => nothing to do!
		if (typeof i !== 'undefined') {
		    var pp_to_country  = from_country_votes[i].Country;
		    var to_country = pp_to_country.replace(/\s+/g,"");

		    var to_country_points = from_country_votes[i].Points;
		    
		    if (typeof to_country_points !== 'undefined') {
			
			//var doc_id = gs.cgiParams["d"];
			//var hyphen_vote_type = doc_id.match(/-(?:T|J)$/);
			var to_country_id = to_country + year;
			var href_to_country = collect_doc_prefix_url + to_country_id;
			
			//$a_to_country = $('<a>').attr("href",href_to_country).append(pp_to_country);
			$a_to_country = $('<a>').attr("href",href_to_country)
			
			$to_country_name = $('<div>').attr("class","voting voting-country").append(pp_to_country).append(":");
			//$to_country_name = $('<div>').attr("class","voting voting-country").append($a_to_country).append(":");
			$to_country_points = $('<div>').attr("class","voting voting-points").append(to_country_points);
		    }
		}

		$a_to_country
		    .append($to_country_name)
		    .append($to_country_points);
		$('#from-country-votes')
		    .append($a_to_country);
	    }
	    $('#from-country-votes')
		.append("<br />");
	}
    }
}
	  

function append_all_from_country_votes()
{
    $('#from-country-votes').html("");

    append_from_country_votes(gs.documentMetadata["FromCountryType"]);
}
